@@ -304,7 +304,9 @@ void child_main(int parent_pipe, struct sockaddr_in *sa_server) {
     int x;
     
 /*    struct passwd* pwent; */
-    
+#ifdef LINUX
+    static int switch_uid = 0;
+#endif
 
     /* Only try to switch if we're running as root */
     if(!getuid()) {
@@ -313,8 +315,24 @@ void child_main(int parent_pipe, struct sockaddr_in *sa_server) {
        /* Now, make absolutely certain we don't have any privileges
         * except those mentioned in the configuration file. */
 	    
+#ifdef LINUX
+       /*
+        * This is very tricky, because we want to switch real
+        * and effective UID while retaining a saved uid.
+        * Don't change this unless you know what you're doing!
+        */
+
+       /* First, make us set-uid. */
+       if (setreuid(user_id, -1) == -1)
+		die(CONF_ERROR,"unable to change uid",stdout);
+       /* Saved uid is now 0. Reset effective uid. */
+       if (seteuid(user_id) == -1)
+		die(CONF_ERROR,"unable to change uid",stdout);
+       switch_uid = 1;
+#else
        if (setuid(user_id) == -1)
 		die(CONF_ERROR,"unable to change uid",stdout);
+#endif
       standalone = 1;
     } 
 
@@ -341,8 +359,25 @@ void child_main(int parent_pipe, struct sockaddr_in *sa_server) {
 	initialize_request();
 	dup2(parent_pipe,0);
 	dup2(parent_pipe,1);
-	if ((csd = recv_fd(parent_pipe)) < 0) {
+#ifdef LINUX
+        /* Switch to root privilige temporarily */
+        if (switch_uid && seteuid(0) < 0) {
+          log_error("child error: seteuid(0)");
+          goto fatal;
+        }
+#endif
+	csd = recv_fd(parent_pipe);
+#ifdef LINUX
+        /* Give up priviliges. */
+        if (switch_uid && seteuid(user_id) < 0) {
+          standalone = 0;
+          die(CONF_ERROR,"unable to change uid",stdout);
+          goto fatal;
+        }
+#endif
+	if (csd < 0) {
 	  log_error("child error: recv_fd()");
+fatal:
 	  close(0);
    	  close(1);
 	  close(sd);
@@ -616,7 +651,7 @@ void standalone_main(int argc, char **argv) {
 extern char *optarg;
 extern int optind;
 
-main(int argc, char *argv[])
+int main(int argc, char *argv[])
 {
     int c;
 
