@@ -144,7 +144,7 @@ typedef int mode_t;
 #define JMP_BUF jmp_buf
 
 #elif defined(LINUX)
-#define NO_PASS
+#define FD_LINUX
 #undef BSD
 #undef NO_KILLPG
 #undef NO_SETSID
@@ -295,7 +295,7 @@ extern char *getenv();
 #endif
 
 /* If we haven't set anything about file descriptor passing, set NO_PASS */
-#if !defined(FD_BSD) && !defined(FD_SYSV) && !defined(NO_PASS)
+#if !defined(FD_BSD) && !defined(FD_SYSV) && !defined(FD_LINUX) && !defined(NO_PASS)
 #define NO_PASS
 #endif
 
